mirror of
https://github.com/c-sooyoung/fold_slice.git
synced 2026-09-17 22:59:07 +09:00
initial commit
This commit is contained in:
@@ -0,0 +1,106 @@
|
||||
%GET_GOLDEN_RATIO_GROUPS use the provided angle to select Ngroups groups containing
|
||||
%the most orthogonal set of projections
|
||||
%
|
||||
% group_ind = get_golden_ratio_groups(theta_all, Ngroups)
|
||||
%
|
||||
% Inputs:
|
||||
% **theta_all - list of angles in degrees
|
||||
% **Ngroups - number of sets to split the angles
|
||||
% *returns*
|
||||
% ++group_ind - cells containing indices for each set
|
||||
|
||||
|
||||
% Academic License Agreement
|
||||
%
|
||||
% Source Code
|
||||
%
|
||||
% Introduction
|
||||
% • This license agreement sets forth the terms and conditions under which the PAUL SCHERRER INSTITUT (PSI), CH-5232 Villigen-PSI, Switzerland (hereafter "LICENSOR")
|
||||
% will grant you (hereafter "LICENSEE") a royalty-free, non-exclusive license for academic, non-commercial purposes only (hereafter "LICENSE") to use the PtychoShelves
|
||||
% computer software program and associated documentation furnished hereunder (hereafter "PROGRAM").
|
||||
%
|
||||
% Terms and Conditions of the LICENSE
|
||||
% 1. LICENSOR grants to LICENSEE a royalty-free, non-exclusive license to use the PROGRAM for academic, non-commercial purposes, upon the terms and conditions
|
||||
% hereinafter set out and until termination of this license as set forth below.
|
||||
% 2. LICENSEE acknowledges that the PROGRAM is a research tool still in the development stage. The PROGRAM is provided without any related services, improvements
|
||||
% or warranties from LICENSOR and that the LICENSE is entered into in order to enable others to utilize the PROGRAM in their academic activities. It is the
|
||||
% LICENSEE’s responsibility to ensure its proper use and the correctness of the results.”
|
||||
% 3. THE PROGRAM IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR
|
||||
% A PARTICULAR PURPOSE AND NONINFRINGEMENT OF ANY PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS. IN NO EVENT SHALL THE LICENSOR, THE AUTHORS OR THE COPYRIGHT
|
||||
% HOLDERS BE LIABLE FOR ANY CLAIM, DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES OR OTHER LIABILITY ARISING FROM, OUT OF OR IN CONNECTION WITH THE PROGRAM OR THE USE
|
||||
% OF THE PROGRAM OR OTHER DEALINGS IN THE PROGRAM.
|
||||
% 4. LICENSEE agrees that it will use the PROGRAM and any modifications, improvements, or derivatives of PROGRAM that LICENSEE may create (collectively,
|
||||
% "IMPROVEMENTS") solely for academic, non-commercial purposes and that any copy of PROGRAM or derivatives thereof shall be distributed only under the same
|
||||
% license as PROGRAM. The terms "academic, non-commercial", as used in this Agreement, mean academic or other scholarly research which (a) is not undertaken for
|
||||
% profit, or (b) is not intended to produce works, services, or data for commercial use, or (c) is neither conducted, nor funded, by a person or an entity engaged
|
||||
% in the commercial use, application or exploitation of works similar to the PROGRAM.
|
||||
% 5. LICENSEE agrees that it shall make the following acknowledgement in any publication resulting from the use of the PROGRAM or any translation of the code into
|
||||
% another computing language:
|
||||
% "Data processing was carried out using the PtychoShelves package developed by the Science IT and the coherent X-ray scattering (CXS) groups, Paul
|
||||
% Scherrer Institut, Switzerland."
|
||||
%
|
||||
% Additionally, any publication using the package, or any translation of the code into another computing language should cite
|
||||
% K. Wakonig, H.-C. Stadler, M. Odstrčil, E.H.R. Tsai, A. Diaz, M. Holler, I. Usov, J. Raabe, A. Menzel, M. Guizar-Sicairos, PtychoShelves, a versatile
|
||||
% high-level framework for high-performance analysis of ptychographic data, J. Appl. Cryst. 53(2) (2020). (doi: 10.1107/S1600576720001776)
|
||||
% and for difference map:
|
||||
% P. Thibault, M. Dierolf, A. Menzel, O. Bunk, C. David, F. Pfeiffer, High-resolution scanning X-ray diffraction microscopy, Science 321, 379–382 (2008).
|
||||
% (doi: 10.1126/science.1158573),
|
||||
% for maximum likelihood:
|
||||
% P. Thibault and M. Guizar-Sicairos, Maximum-likelihood refinement for coherent diffractive imaging, New J. Phys. 14, 063004 (2012).
|
||||
% (doi: 10.1088/1367-2630/14/6/063004),
|
||||
% for LSQ-ML:
|
||||
% M. Odstrčil, A. Menzel, and M. Guizar-Sicairos, Iterative least-squares solver for generalized maximum-likelihood ptychography, Opt. Express 26(3), 3108 (2018).
|
||||
% (doi: 10.1364/OE.26.003108),
|
||||
% for mixed coherent modes:
|
||||
% P. Thibault and A. Menzel, Reconstructing state mixtures from diffraction measurements, Nature 494, 68–71 (2013). (doi: 10.1038/nature11806),
|
||||
% and/or for multislice:
|
||||
% E. H. R. Tsai, I. Usov, A. Diaz, A. Menzel, and M. Guizar-Sicairos, X-ray ptychography with extended depth of field, Opt. Express 24, 29089–29108 (2016).
|
||||
% (doi: 10.1364/OE.24.029089),
|
||||
% and/or for OPRP:
|
||||
% M. Odstrcil, P. Baksh, S. A. Boden, R. Card, J. E. Chad, J. G. Frey, W. S. Brocklesby, Ptychographic coherent diffractive imaging with orthogonal probe relaxation.
|
||||
% Opt. Express 24.8 (8360-8369) 2016. (doi: 10.1364/OE.24.008360).
|
||||
% 6. Except for the above-mentioned acknowledgment, LICENSEE shall not use the PROGRAM title or the names or logos of LICENSOR, nor any adaptation thereof, nor the
|
||||
% names of any of its employees or laboratories, in any advertising, promotional or sales material without prior written consent obtained from LICENSOR in each case.
|
||||
% 7. Ownership of all rights, including copyright in the PROGRAM and in any material associated therewith, shall at all times remain with LICENSOR, and LICENSEE
|
||||
% agrees to preserve same. LICENSEE agrees not to use any portion of the PROGRAM or of any IMPROVEMENTS in any machine-readable form outside the PROGRAM, nor to
|
||||
% make any copies except for its internal use, without prior written consent of LICENSOR. LICENSEE agrees to place the following copyright notice on any such copies:
|
||||
% © All rights reserved. PAUL SCHERRER INSTITUT, Switzerland, Laboratory for Macromolecules and Bioimaging, 2017.
|
||||
% 8. The LICENSE shall not be construed to confer any rights upon LICENSEE by implication or otherwise except as specifically set forth herein.
|
||||
% 9. DISCLAIMER: LICENSEE shall be aware that Phase Focus Limited of Sheffield, UK has an international portfolio of patents and pending applications which relate
|
||||
% to ptychography and that the PROGRAM may be capable of being used in circumstances which may fall within the claims of one or more of the Phase Focus patents,
|
||||
% in particular of patent with international application number PCT/GB2005/001464. The LICENSOR explicitly declares not to indemnify the users of the software
|
||||
% in case Phase Focus or any other third party will open a legal action against the LICENSEE due to the use of the program.
|
||||
% 10. This Agreement shall be governed by the material laws of Switzerland and any dispute arising out of this Agreement or use of the PROGRAM shall be brought before
|
||||
% the courts of Zürich, Switzerland.
|
||||
|
||||
|
||||
function group_ind = get_golden_ratio_groups(theta_all, Ngroups)
|
||||
Nangles = length(theta_all);
|
||||
|
||||
if all(theta_all == theta_all(1))
|
||||
group_ind = {};
|
||||
for ll = 1:Ngroups
|
||||
ind = 1+(ll-1)*ceil(Nangles/Ngroups):min(Nangles, (ll)*ceil(Nangles/Ngroups));
|
||||
group_ind{end+1} = (ind);
|
||||
end
|
||||
return
|
||||
end
|
||||
|
||||
theta_aux = theta_all;
|
||||
Nangles = length(theta_all);
|
||||
start_index = 200;
|
||||
|
||||
for ll = 1:Nangles
|
||||
% golden ratio angle selector
|
||||
gold_angle = mod((ll + start_index) * (1+sqrt(5))/2* 360, 360);
|
||||
theta = theta_aux(math.argmin(abs(theta_aux-gold_angle)));
|
||||
theta_aux = setdiff(theta_aux, theta);
|
||||
index(ll) = find(theta_all== theta);
|
||||
end
|
||||
|
||||
group_ind = {};
|
||||
for ll = 1:Ngroups
|
||||
ind = 1+(ll-1)*ceil(Nangles/Ngroups):min(Nangles, (ll)*ceil(Nangles/Ngroups));
|
||||
group_ind{end+1} = index(ind);
|
||||
end
|
||||
end
|
||||
@@ -0,0 +1,203 @@
|
||||
/*
|
||||
SET_TO_BLOCK_GPU Set complex views to a large complex object without memory copy of the large volume
|
||||
|
||||
set_to_block_gpu(volData, volData_block, uint16(ind));
|
||||
|
||||
Inputs:
|
||||
**volData - 3D array, single precision
|
||||
**volData_block - 3D array, that will be written into the volData
|
||||
**ind - 1D array, uint16, indices into which lines of volData will be volData_block written
|
||||
Outputs:
|
||||
none, results are written to volData directly
|
||||
|
||||
Recompile:
|
||||
mexcuda -output +engines/+GPU/+shared/private/set_views_gpu_mex +engines/+GPU/+shared/private/set_views_gpu_mex.cu
|
||||
*/
|
||||
|
||||
#include "mex.h"
|
||||
#include "gpu/mxGPUArray.h"
|
||||
#include <math.h>
|
||||
#include <stdio.h>
|
||||
|
||||
typedef const unsigned int cuint;
|
||||
typedef const uint16_T cuint16;
|
||||
|
||||
// unfortunatelly ~10800 is the maximum of const memory
|
||||
const unsigned int MAX_IND_READ = 10800;
|
||||
static const unsigned MAX_IND_READ_DEV = MAX_IND_READ;
|
||||
__constant__ uint16_T gC_ind[MAX_IND_READ_DEV];
|
||||
|
||||
|
||||
int checkLastError(char * msg)
|
||||
{
|
||||
cudaError_t cudaStatus = cudaGetLastError();
|
||||
if (cudaStatus != cudaSuccess) {
|
||||
char err[512];
|
||||
sprintf(err, "setprojection failed \n %s: %s. \n", msg, cudaGetErrorString(cudaStatus));
|
||||
mexPrintf(err);
|
||||
return 1;
|
||||
}
|
||||
return 0;
|
||||
}
|
||||
|
||||
|
||||
/*
|
||||
* Device code
|
||||
*/
|
||||
|
||||
|
||||
__global__ void addToArray_c( float2 const * sarray, float2 * larray, cuint Nx,cuint Ny, cuint Npos) {
|
||||
// Location in a 3D matrix
|
||||
int idx= blockIdx.x * blockDim.x + threadIdx.x;
|
||||
int idy= blockIdx.y * blockDim.y + threadIdx.y;
|
||||
if ( idx < Nx & idy < Ny ) {
|
||||
int idz, id_large, id_small;
|
||||
for(idz = 0; idz < Npos; idz++)
|
||||
{
|
||||
id_large = idx + Nx*idy + Nx*Ny*(gC_ind[idz]-1); // go only through some of the indices
|
||||
id_small = idx + Nx*idy + Nx*Ny*idz;
|
||||
|
||||
|
||||
larray[id_large].x = sarray[ id_small ].x;
|
||||
larray[id_large].y = sarray[ id_small ].y;
|
||||
|
||||
}
|
||||
}
|
||||
}
|
||||
|
||||
|
||||
void mexFunction(int nlhs, mxArray *plhs[],
|
||||
int nrhs, const mxArray *prhs[])
|
||||
{
|
||||
|
||||
|
||||
int i ;
|
||||
char const * const errId = "parallel:gpu:mexGPUExample:InvalidInput";
|
||||
char const * const errMsg = "Invalid input to MEX file.";
|
||||
|
||||
// Check for proper number of arguments.
|
||||
if (nrhs != 3)
|
||||
mexErrMsgTxt("Three input arguments required");
|
||||
|
||||
|
||||
// Input must be of type single.
|
||||
for (i=0; i < 2; i++) {
|
||||
if ( !mxIsGPUArray(prhs[i]) ){
|
||||
printf("Input %d is not cell array\n",i+1);
|
||||
mexErrMsgIdAndTxt("MexError:ptycho","Inputs must be of correct type.");
|
||||
}
|
||||
}
|
||||
// Input must be of type int16.
|
||||
for (i=2; i<3; i++){
|
||||
if (mxIsUint16(prhs[i]) != 1){
|
||||
printf("Input %d is not integer\n",i+1);
|
||||
mexErrMsgIdAndTxt("MexError:ptycho","Inputs must be of correct type uint16.");
|
||||
}
|
||||
}
|
||||
|
||||
// It cannot be one-dimensional
|
||||
if(mxGetNumberOfDimensions(prhs[0]) > 3) {
|
||||
printf("The 1st input argument must have at least three dimensions.");
|
||||
mexErrMsgIdAndTxt("MexError:ptycho","wrong number of dimensions");
|
||||
}
|
||||
// It cannot be more than 3-dimensional
|
||||
if(mxGetNumberOfDimensions(prhs[0]) > 3) {
|
||||
printf("The 1st input argument must have at most three dimensions.");
|
||||
mexErrMsgIdAndTxt("MexError:ptycho","wrong number of dimensions");
|
||||
}
|
||||
|
||||
// Check that arrays are complex
|
||||
|
||||
|
||||
const mxGPUArray * m_sarray = mxGPUCreateFromMxArray(prhs[1]);
|
||||
if ((mxGPUGetClassID(m_sarray) != mxSINGLE_CLASS)) {
|
||||
mexPrintf("m_sarray\n");
|
||||
mexErrMsgIdAndTxt(errId, errMsg);
|
||||
}
|
||||
|
||||
// write results directly to the output array
|
||||
mxGPUArray * m_larray = const_cast<mxGPUArray*>(mxGPUCreateFromMxArray(prhs[0])); // mxGPUCopyFromMxArray(prhs[1]);
|
||||
if ((mxGPUGetClassID(m_larray) != mxSINGLE_CLASS)) {
|
||||
mexPrintf("m_larray\n");
|
||||
mexErrMsgIdAndTxt(errId, errMsg);
|
||||
}
|
||||
|
||||
if (mxGPUGetComplexity(m_larray) != mxCOMPLEX) {
|
||||
mexPrintf("m_larray complexity has to be the COMPLEX \n");
|
||||
mexErrMsgIdAndTxt(errId, errMsg);
|
||||
}
|
||||
|
||||
|
||||
if (mxGPUGetComplexity(m_larray) != mxGPUGetComplexity(m_sarray)) {
|
||||
mexPrintf("m_larray and m_sarray complexity has to be the same \n");
|
||||
mexErrMsgIdAndTxt(errId, errMsg);
|
||||
}
|
||||
|
||||
|
||||
|
||||
const mxGPUArray * m_ind = mxGPUCreateFromMxArray(prhs[2]);
|
||||
if ((mxGPUGetClassID(m_ind) != mxUINT16_CLASS)) {
|
||||
mexPrintf("m_ind\n");
|
||||
mexErrMsgIdAndTxt(errId, errMsg);
|
||||
}
|
||||
const uint16_T * p_ind = (uint16_T *)mxGPUGetDataReadOnly(m_ind);
|
||||
|
||||
|
||||
|
||||
// Get dimension of probe and object
|
||||
const unsigned int Ndims = (unsigned int)mxGPUGetNumberOfDimensions(m_sarray);
|
||||
const mwSize * Np_l = mxGPUGetDimensions(m_larray);
|
||||
const mwSize * Np_s = mxGPUGetDimensions(m_sarray);
|
||||
const unsigned int Npos = mxGPUGetNumberOfElements(m_ind);
|
||||
|
||||
|
||||
|
||||
|
||||
if (Ndims == 3 && Npos > Np_s[2]) {
|
||||
printf("wrong size of update / positions %i", Ndims);
|
||||
mexErrMsgIdAndTxt("MexError:ptycho","wrong size of update / positions");
|
||||
}
|
||||
|
||||
|
||||
cudaMemcpyToSymbol(gC_ind, p_ind, Npos*sizeof(uint16_T), 0, cudaMemcpyHostToDevice);
|
||||
checkLastError("after cudaMemcpyToSymbol pos");
|
||||
|
||||
|
||||
// Choose a reasonably sized number of threads in each dimension for the block.
|
||||
int const threadsPerBlockEachDim = 32;
|
||||
// Compute the thread block and grid sizes based on the board dimensions.
|
||||
int const blocksPerGrid_M = (Np_s[0] + threadsPerBlockEachDim - 1) / threadsPerBlockEachDim;
|
||||
int const blocksPerGrid_N = (Np_s[1] + threadsPerBlockEachDim - 1) / threadsPerBlockEachDim;
|
||||
int const blocksPerGrid_O = 1;
|
||||
|
||||
dim3 const dimBlock(blocksPerGrid_M, blocksPerGrid_N, blocksPerGrid_O);
|
||||
dim3 const dimThread(threadsPerBlockEachDim, threadsPerBlockEachDim, 1);
|
||||
|
||||
checkLastError("after dimThread");
|
||||
|
||||
checkLastError("after cudaMemcpyToSymbol");
|
||||
|
||||
// ================== call the right kernel ===================
|
||||
|
||||
const float2 * p_sarray = (float2 *)mxGPUGetDataReadOnly(m_sarray);
|
||||
float2 * p_larray = (float2 *)mxGPUGetData(m_larray);
|
||||
addToArray_c<<<dimBlock, dimThread>>>(p_sarray,p_larray, Np_l[0],Np_l[1], Npos);
|
||||
|
||||
|
||||
checkLastError("after kernel");
|
||||
|
||||
mxGPUDestroyGPUArray(m_larray);
|
||||
mxGPUDestroyGPUArray(m_ind);
|
||||
|
||||
cudaThreadSynchronize();
|
||||
|
||||
// plhs[0] = mxGPUCreateMxArrayOnGPU(m_larray);
|
||||
mxGPUDestroyGPUArray(m_sarray);
|
||||
|
||||
|
||||
return;
|
||||
}
|
||||
|
||||
|
||||
|
||||
|
||||
Reference in New Issue
Block a user